#!/usr/bin/python
'''
LED Matrix
===========
- configure LED matrix via GUI, generate Arduino .ino file
- by AM 2017 (andreas.moeglich@uni-bayreuth.de)
'''

from PyQt4 import QtCore, QtGui
from functools import partial
import re
import sys

try:
  _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
  def _fromUtf8(s):
    return s

try:
  _encoding = QtGui.QApplication.UnicodeUTF8
  def _translate(context, text, disambig):
    return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
  def _translate(context, text, disambig):
    return QtGui.QApplication.translate(context, text, disambig)

class myLineEdit(QtGui.QLineEdit):
  def __init__(self,parent,currled=0,field=0,mainUI=None):
    super(myLineEdit,self).__init__(parent)
   # QtGui.QLineEdit.__init__(self,parent)
    self.currled=currled
    self.field=field
    self.mainUI=mainUI
  
  # capture arrow keys
  def event(self, event):
    if (event.type()==QtCore.QEvent.KeyPress) and (event.key()==QtCore.Qt.Key_Up):
      if (self.field>3):
        self.mainUI.leds[self.currled]['qedit'][self.field-4].setFocus()
        self.mainUI.leds[self.currled]['qedit'][self.field-4].selectAll()
        self.mainUI.scrollArea.ensureWidgetVisible(self.mainUI.leds[self.currled]['qedit'][self.field-4])
      elif(self.currled):
        # extra check whether field is enabled
        if (self.mainUI.leds[self.currled-1]['qedit'][self.field+4].isEnabled()):
          self.mainUI.leds[self.currled-1]['qedit'][self.field+4].setFocus()
          self.mainUI.leds[self.currled-1]['qedit'][self.field+4].selectAll()
          self.mainUI.scrollArea.ensureWidgetVisible(self.mainUI.leds[self.currled-1]['qedit'][self.field+4])
        else:
          self.mainUI.leds[self.currled-1]['qedit'][self.field].setFocus()
          self.mainUI.leds[self.currled-1]['qedit'][self.field].selectAll()
          self.mainUI.scrollArea.ensureWidgetVisible(self.mainUI.leds[self.currled-1]['qedit'][self.field])
      return True
    elif (event.type()==QtCore.QEvent.KeyPress) and (event.key()==QtCore.Qt.Key_Down):
      if (self.field<4):
        # extra check whether field is enabled
        if (self.mainUI.leds[self.currled]['qedit'][self.field+4].isEnabled()):
          self.mainUI.leds[self.currled]['qedit'][self.field+4].setFocus()
          self.mainUI.leds[self.currled]['qedit'][self.field+4].selectAll()
          self.mainUI.scrollArea.ensureWidgetVisible(self.mainUI.leds[self.currled]['qedit'][self.field+4])
        elif(self.currled<self.mainUI.LED_NUMBER-1):
          self.mainUI.leds[self.currled+1]['qedit'][self.field].setFocus()
          self.mainUI.leds[self.currled+1]['qedit'][self.field].selectAll()
          self.mainUI.scrollArea.ensureWidgetVisible(self.mainUI.leds[self.currled+1]['qedit'][self.field])
      elif(self.currled<self.mainUI.LED_NUMBER-1):
        self.mainUI.leds[self.currled+1]['qedit'][self.field-4].setFocus()
        self.mainUI.leds[self.currled+1]['qedit'][self.field-4].selectAll()
        self.mainUI.scrollArea.ensureWidgetVisible(self.mainUI.leds[self.currled+1]['qedit'][self.field-4])
      return True

    return QtGui.QLineEdit.event(self, event)
    
class PaintArea(QtGui.QWidget):
  def __init__(self,numled=64,mainUI=None):
    super(PaintArea,self).__init__()
    self.mainUI=mainUI
    self.CIRCLE_RADIUS=int(40*(1.0*mainUI.DPI_SCALING/2))
    self.CIRCLE_BUFFER=int(5*(1.0*mainUI.DPI_SCALING/2))
    self.ledcolor=[[255,255,255] for i in range(numled)]
    self.show()

  def paintEvent(self, event):
    self.qp = QtGui.QPainter(self)
    # don't need explicit begin() call as constructor does this already
    #self.qp.begin(self)

    self.mybrush=QtGui.QBrush(QtCore.Qt.red)
    self.mybrush.setColor(QtCore.Qt.black)
    self.qp.setBrush(self.mybrush)
    self.mypen=QtGui.QPen()
    self.mypen.setWidth(1.5)
    self.mypen.setColor(QtCore.Qt.black)
    self.qp.setPen(self.mypen)
    self.qp.drawRoundedRect(0, 0, 8*self.CIRCLE_RADIUS+9*self.CIRCLE_BUFFER,\
      8*self.CIRCLE_RADIUS+9*self.CIRCLE_BUFFER, 10.0, 10.0)
    
    self.mypen.setWidth(0.5)
    self.mypen.setColor(QtCore.Qt.white)
    self.qp.setPen(self.mypen)
    
    for row in range(8):
      for col in range(8):
        self.mybrush.setColor(QtGui.QColor(*self.ledcolor[row+col*8]))
        self.qp.setBrush(self.mybrush)
        self.qp.drawEllipse(self.CIRCLE_BUFFER+row*(self.CIRCLE_RADIUS+self.CIRCLE_BUFFER),\
          self.CIRCLE_BUFFER+col*(self.CIRCLE_RADIUS+self.CIRCLE_BUFFER),self.CIRCLE_RADIUS,\
          self.CIRCLE_RADIUS)

    self.qp.end()
    
  def update_color(self,number=0,rgb=[0,0,0]):
    self.ledcolor[number]=rgb
    self.update()
    
  def mousePressEvent(self, QMouseEvent):
    # get x,y coords of click
    coordx=QMouseEvent.pos().x()-self.CIRCLE_BUFFER
    coordy=QMouseEvent.pos().y()-self.CIRCLE_BUFFER
    
    # calculate which LED was clicked
    row=int(coordy/(self.CIRCLE_RADIUS+self.CIRCLE_BUFFER))
    col=int(coordx/(self.CIRCLE_RADIUS+self.CIRCLE_BUFFER))
    if((row<8) and (col<8)):
      clickedLED=col+row*8
      self.mainUI.select_LED(clickedLED)

class Ui_MainWindow(object):
  def setupUi(self, MainWindow,DPI_SCALING=1):
    self.DPI_SCALING=DPI_SCALING
    # set up main window
    MainWindow.setObjectName(_fromUtf8("MainWindow"))
    self.WINDOW_WIDTH=int(800*(1.0*self.DPI_SCALING/2))
    self.WINDOW_HEIGHT=int(600*(1.0*self.DPI_SCALING/2))
    MainWindow.resize(self.WINDOW_WIDTH,self.WINDOW_HEIGHT)
    
    self.centralwidget = QtGui.QWidget(MainWindow)
    self.centralwidget.setObjectName(_fromUtf8("centralwidget"))
    
    self.horizontalLayout1=QtGui.QHBoxLayout(self.centralwidget)
    self.verticalLayout1 = QtGui.QVBoxLayout()
    self.horizontalLayout1.addLayout(self.verticalLayout1)
    
    # build left panel
    self.scrollArea = QtGui.QScrollArea(self.centralwidget)
    self.scrollArea.setMinimumSize(QtCore.QSize(self.WINDOW_WIDTH,self.WINDOW_HEIGHT))
    self.scrollArea.setMaximumSize(QtCore.QSize(self.WINDOW_WIDTH,16777215))
    self.scrollArea.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
    self.scrollArea.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
    self.scrollArea.setWidgetResizable(True)
    
    self.verticalLayout1.addWidget(self.scrollArea)

    self.ledwidget = QtGui.QWidget(self.scrollArea)
    self.ledwidget.setGeometry(QtCore.QRect(0, 0,int(780*(1.0*self.DPI_SCALING/2)),\
      int(800*(1.0*self.DPI_SCALING/2))))
    self.ledwidget.setObjectName(_fromUtf8("ledwidget"))
    
    self.verticalLayout = QtGui.QVBoxLayout(self.ledwidget)
    self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
    self.verticalLayout.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)

    # set up validators
    self.valid_color=QtGui.QIntValidator(0,255)
    self.valid_time=QtGui.QIntValidator()
    self.valid_time.setBottom(0)
    
    # set up ressource
    # size definitions for DPI scaling
    xoff=int(10*(1.0*self.DPI_SCALING/2))
    yoff=int(10*(1.0*self.DPI_SCALING/2))
    # for qlabels
    self.CONTAINER_WIDTH=int(111*(1.0*self.DPI_SCALING/2))
    self.CONTAINER_HEIGHT=int(41*(1.0*self.DPI_SCALING/2))
    self.LABEL_HEIGHT=int(27*(1.0*self.DPI_SCALING/2))
    self.LABEL_WIDTH1=int(91*(1.0*self.DPI_SCALING/2))
    self.LABEL_WIDTH2=int(31*(1.0*self.DPI_SCALING/2))
    self.XOFFSET=[int(j*(1.0*self.DPI_SCALING/2)) for j in (0,110,230,350,520)]
    self.YOFFSET=[int(j*(1.0*self.DPI_SCALING/2)) for j in (60,110)]
    # for radio buttons
    self.RADIO_CONTAINER_WIDTH=int(270*(1.0*self.DPI_SCALING/2))
    self.RADIO_CONTAINER_HEIGHT=int(50*(1.0*self.DPI_SCALING/2))
    self.RADIO_WIDTH=int(120*(1.0*self.DPI_SCALING/2))
    self.RADIO_HEIGHT=int(31*(1.0*self.DPI_SCALING/2))
    self.RADIO_XOFFSET=int(135*(1.0*self.DPI_SCALING/2))
    # for copy elements
    self.COPY_XOFFSET=[int(j*(1.0*self.DPI_SCALING/2)) for j in (400,510)]
    self.COPY_CHECK_WIDTH=int(50*(1.0*self.DPI_SCALING/2))
    self.COPY_CHECK_HEIGHT=self.COPY_CHECK_WIDTH
    self.COPY_WIDTH=int(100*(1.0*self.DPI_SCALING/2))
    self.COPY_HEIGHT=int(40*(1.0*self.DPI_SCALING/2))
    # for qlineedits
    self.QLINEEDIT_XOFFSET=[int(j*(1.0*self.DPI_SCALING/2)) for j in (140,260,380,610)]
    self.QLINEEDIT_YOFFSET=[int(j*(1.0*self.DPI_SCALING/2)) for j in (50,100)]
    self.QLINEEDIT_WIDTH1=int(71*(1.0*self.DPI_SCALING/2))
    self.QLINEEDIT_WIDTH2=int(111*(1.0*self.DPI_SCALING/2))
    self.QLINEEDIT_HEIGHT=int(41*(1.0*self.DPI_SCALING/2))
    # for colored boxes
    self.CBOX_XOFFSET=int(470*(1.0*self.DPI_SCALING/2))
    self.CBOX_YOFFSET=[int(j*(1.0*self.DPI_SCALING/2)) for j in (50,100)]
    self.CBOX_WIDTH=int(40*(1.0*self.DPI_SCALING/2))
    self.CBOX_HEIGHT=self.CBOX_WIDTH#
    
    # set up entries
    self.LED_NUMBER=64
    self.leds=[]
    for currled in range(self.LED_NUMBER):
      # initialize and set offset
      self.leds.append({})
      
      self.leds[-1]['container']=QtGui.QGroupBox(self.ledwidget)
      self.leds[-1]['container'].setMinimumSize(QtCore.QSize(int(740*(1.0*self.DPI_SCALING/2)),\
        int(160*(1.0*self.DPI_SCALING/2))))
      self.leds[-1]['container'].setMaximumSize(QtCore.QSize(int(740*(1.0*self.DPI_SCALING/2)),\
        int(160*(1.0*self.DPI_SCALING/2))))
      
      # build elements    
      # set up labels
      self.leds[-1]['qlabels']=[]
      self.leds[-1]['qlabels'].append(QtGui.QLabel(self.leds[-1]['container']))
      self.leds[-1]['qlabels'][-1].setGeometry(QtCore.QRect(xoff,yoff,self.CONTAINER_WIDTH,self.CONTAINER_HEIGHT))
      self.leds[-1]['qlabels'][-1].setText(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:12pt;\">LED "+str(currled)+"</span></p></body></html>", None))
      
      # 1st row
      self.leds[-1]['qlabels'].append(QtGui.QLabel(self.leds[-1]['container']))
      self.leds[-1]['qlabels'][-1].setGeometry(QtCore.QRect(xoff+self.XOFFSET[0],yoff+self.YOFFSET[0],\
        self.LABEL_WIDTH1,self.LABEL_HEIGHT))
      self.leds[-1]['qlabels'][-1].setText(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">Color 1</span></p></body></html>", None))
      self.leds[-1]['qlabels'].append(QtGui.QLabel(self.leds[-1]['container']))
      self.leds[-1]['qlabels'][-1].setGeometry(QtCore.QRect(xoff+self.XOFFSET[1],yoff+self.YOFFSET[0],\
        self.LABEL_WIDTH2,self.LABEL_HEIGHT))
      self.leds[-1]['qlabels'][-1].setText(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">R</span></p></body></html>", None))
      self.leds[-1]['qlabels'].append(QtGui.QLabel(self.leds[-1]['container']))
      self.leds[-1]['qlabels'][-1].setGeometry(QtCore.QRect(xoff+self.XOFFSET[2],yoff+self.YOFFSET[0],\
        self.LABEL_WIDTH2,self.LABEL_HEIGHT))
      self.leds[-1]['qlabels'][-1].setText(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">G</span></p></body></html>", None))
      self.leds[-1]['qlabels'].append(QtGui.QLabel(self.leds[-1]['container']))
      self.leds[-1]['qlabels'][-1].setGeometry(QtCore.QRect(xoff+self.XOFFSET[3],yoff+self.YOFFSET[0],\
        self.LABEL_WIDTH2,self.LABEL_HEIGHT))
      self.leds[-1]['qlabels'][-1].setText(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">B</span></p></body></html>", None))
      self.leds[-1]['qlabels'].append(QtGui.QLabel(self.leds[-1]['container']))
      self.leds[-1]['qlabels'][-1].setGeometry(QtCore.QRect(xoff+self.XOFFSET[4],yoff+self.YOFFSET[0],\
        self.LABEL_WIDTH1,self.LABEL_HEIGHT))
      self.leds[-1]['qlabels'][-1].setText(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">Time 1</span></p></body></html>", None))

      # 2nd row
      self.leds[-1]['qlabels'].append(QtGui.QLabel(self.leds[-1]['container']))
      self.leds[-1]['qlabels'][-1].setGeometry(QtCore.QRect(xoff+self.XOFFSET[0],yoff+self.YOFFSET[1],\
        self.LABEL_WIDTH1,self.LABEL_HEIGHT))
      self.leds[-1]['qlabels'][-1].setText(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">Color 2</span></p></body></html>", None))
      self.leds[-1]['qlabels'].append(QtGui.QLabel(self.leds[-1]['container']))
      self.leds[-1]['qlabels'][-1].setGeometry(QtCore.QRect(xoff+self.XOFFSET[1],yoff+self.YOFFSET[1],\
        self.LABEL_WIDTH2,self.LABEL_HEIGHT))
      self.leds[-1]['qlabels'][-1].setText(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">R</span></p></body></html>", None))
      self.leds[-1]['qlabels'].append(QtGui.QLabel(self.leds[-1]['container']))
      self.leds[-1]['qlabels'][-1].setGeometry(QtCore.QRect(xoff+self.XOFFSET[2],yoff+self.YOFFSET[1],\
        self.LABEL_WIDTH2,self.LABEL_HEIGHT))
      self.leds[-1]['qlabels'][-1].setText(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">G</span></p></body></html>", None))
      self.leds[-1]['qlabels'].append(QtGui.QLabel(self.leds[-1]['container']))
      self.leds[-1]['qlabels'][-1].setGeometry(QtCore.QRect(xoff+self.XOFFSET[3],yoff+self.YOFFSET[1],\
        self.LABEL_WIDTH2,self.LABEL_HEIGHT))
      self.leds[-1]['qlabels'][-1].setText(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">B</span></p></body></html>", None))
      self.leds[-1]['qlabels'].append(QtGui.QLabel(self.leds[-1]['container']))
      self.leds[-1]['qlabels'][-1].setGeometry(QtCore.QRect(xoff+self.XOFFSET[4],yoff+self.YOFFSET[1],\
        self.LABEL_WIDTH1,self.LABEL_HEIGHT))
      self.leds[-1]['qlabels'][-1].setText(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">Time 2</span></p></body></html>", None))

      
      # set up radio buttons
      self.leds[-1]['qradio']=[]
      self.leds[-1]['qradio'].append(QtGui.QGroupBox(self.leds[-1]['container']))
      self.leds[-1]['qradio'][-1].setGeometry(QtCore.QRect(xoff+self.XOFFSET[1],yoff,self.RADIO_CONTAINER_WIDTH,self.RADIO_CONTAINER_HEIGHT))
      
      self.leds[-1]['qradio'].append(QtGui.QRadioButton(self.leds[-1]['qradio'][0]))
      self.leds[-1]['qradio'][-1].setGeometry(QtCore.QRect(xoff/2, yoff/2, self.RADIO_WIDTH, self.RADIO_HEIGHT))
      self.leds[-1]['qradio'][-1].setChecked(True)
      self.leds[-1]['qradio'][-1].setText(_translate("MainWindow", "box car", None))
      self.leds[-1]['qradio'].append(QtGui.QRadioButton(self.leds[-1]['qradio'][0]))
      self.leds[-1]['qradio'][-1].setGeometry(QtCore.QRect(self.RADIO_XOFFSET, yoff/2, self.RADIO_WIDTH, self.RADIO_HEIGHT))
      self.leds[-1]['qradio'][-1].setChecked(False)
      self.leds[-1]['qradio'][-1].setText(_translate("MainWindow", "sinusoid", None))
      
      # set up check box and copy button
      self.leds[-1]['check']=[]
      self.leds[-1]['check'].append(QtGui.QCheckBox(self.leds[-1]['container']))
      self.leds[-1]['check'][0].setGeometry(QtCore.QRect(xoff+self.COPY_XOFFSET[1],yoff,\
        self.COPY_CHECK_WIDTH,self.COPY_CHECK_HEIGHT))
      self.leds[-1]['check'][0].setChecked(False)
      
      self.leds[-1]['check'].append(QtGui.QPushButton(self.leds[-1]['container']))
      self.leds[-1]['check'][1].setGeometry(QtCore.QRect(xoff+self.COPY_XOFFSET[0],yoff+5,\
        self.COPY_WIDTH,self.COPY_HEIGHT))
      self.leds[-1]['check'][1].setText('Copy')

      # set up qedits
      # 1st row
      self.leds[-1]['qedit']=[]
      self.leds[-1]['qedit'].append(myLineEdit(self.leds[-1]['container'],currled,0,self))
      self.leds[-1]['qedit'][-1].setGeometry(QtCore.QRect(xoff+self.QLINEEDIT_XOFFSET[0],\
        yoff+self.QLINEEDIT_YOFFSET[0],self.QLINEEDIT_WIDTH1,self.QLINEEDIT_HEIGHT))
      self.leds[-1]['qedit'][-1].setMaxLength(4)
      self.leds[-1]['qedit'][-1].setText(str(0))
      self.leds[-1]['qedit'][-1].setValidator(self.valid_color)
      self.leds[-1]['qedit'].append(myLineEdit(self.leds[-1]['container'],currled,1,self))
      self.leds[-1]['qedit'][-1].setGeometry(QtCore.QRect(xoff+self.QLINEEDIT_XOFFSET[1],\
        yoff+self.QLINEEDIT_YOFFSET[0],self.QLINEEDIT_WIDTH1,self.QLINEEDIT_HEIGHT))
      self.leds[-1]['qedit'][-1].setMaxLength(4)
      self.leds[-1]['qedit'][-1].setText(str(0))
      self.leds[-1]['qedit'][-1].setValidator(self.valid_color)
      self.leds[-1]['qedit'].append(myLineEdit(self.leds[-1]['container'],currled,2,self))
      self.leds[-1]['qedit'][-1].setGeometry(QtCore.QRect(xoff+self.QLINEEDIT_XOFFSET[2],\
        yoff+self.QLINEEDIT_YOFFSET[0],self.QLINEEDIT_WIDTH1,self.QLINEEDIT_HEIGHT))
      self.leds[-1]['qedit'][-1].setMaxLength(4)
      self.leds[-1]['qedit'][-1].setText(str(0))
      self.leds[-1]['qedit'][-1].setValidator(self.valid_color)
      self.leds[-1]['qedit'].append(myLineEdit(self.leds[-1]['container'],currled,3,self))
      self.leds[-1]['qedit'][-1].setGeometry(QtCore.QRect(xoff+self.QLINEEDIT_XOFFSET[3],\
        yoff+self.QLINEEDIT_YOFFSET[0],self.QLINEEDIT_WIDTH2,self.QLINEEDIT_HEIGHT))
      self.leds[-1]['qedit'][-1].setMaxLength(8)
      self.leds[-1]['qedit'][-1].setText(str(5000))
      self.leds[-1]['qedit'][-1].setValidator(self.valid_time)

      # 2nd row
      self.leds[-1]['qedit'].append(myLineEdit(self.leds[-1]['container'],currled,4,self))
      self.leds[-1]['qedit'][-1].setGeometry(QtCore.QRect(xoff+self.QLINEEDIT_XOFFSET[0],\
        yoff+self.QLINEEDIT_YOFFSET[1],self.QLINEEDIT_WIDTH1,self.QLINEEDIT_HEIGHT))
      self.leds[-1]['qedit'][-1].setMaxLength(4)
      self.leds[-1]['qedit'][-1].setText(str(0))
      self.leds[-1]['qedit'][-1].setValidator(self.valid_color)
      self.leds[-1]['qedit'].append(myLineEdit(self.leds[-1]['container'],currled,5,self))
      self.leds[-1]['qedit'][-1].setGeometry(QtCore.QRect(xoff+self.QLINEEDIT_XOFFSET[1],\
        yoff+self.QLINEEDIT_YOFFSET[1],self.QLINEEDIT_WIDTH1,self.QLINEEDIT_HEIGHT))
      self.leds[-1]['qedit'][-1].setMaxLength(4)
      self.leds[-1]['qedit'][-1].setText(str(0))
      self.leds[-1]['qedit'][-1].setValidator(self.valid_color)
      self.leds[-1]['qedit'].append(myLineEdit(self.leds[-1]['container'],currled,6,self))
      self.leds[-1]['qedit'][-1].setGeometry(QtCore.QRect(xoff+self.QLINEEDIT_XOFFSET[2],\
        yoff+self.QLINEEDIT_YOFFSET[1],self.QLINEEDIT_WIDTH1,self.QLINEEDIT_HEIGHT))
      self.leds[-1]['qedit'][-1].setMaxLength(4)
      self.leds[-1]['qedit'][-1].setText(str(0))
      self.leds[-1]['qedit'][-1].setValidator(self.valid_color)
      self.leds[-1]['qedit'].append(myLineEdit(self.leds[-1]['container'],currled,7,self))
      self.leds[-1]['qedit'][-1].setGeometry(QtCore.QRect(xoff+self.QLINEEDIT_XOFFSET[3],\
        yoff+self.QLINEEDIT_YOFFSET[1],self.QLINEEDIT_WIDTH2,self.QLINEEDIT_HEIGHT))
      self.leds[-1]['qedit'][-1].setMaxLength(8)
      self.leds[-1]['qedit'][-1].setText(str(1000))
      self.leds[-1]['qedit'][-1].setValidator(self.valid_time)

      # set up color boxes
      # 1st row
      self.leds[-1]['cbox']=[]
      self.leds[-1]['cbox'].append(QtGui.QGroupBox(self.leds[-1]['container']))
      self.leds[-1]['cbox'][-1].setGeometry(QtCore.QRect(xoff+self.CBOX_XOFFSET,\
        yoff+self.CBOX_YOFFSET[0],self.CBOX_WIDTH,self.CBOX_HEIGHT))
      self.leds[-1]['cbox'][-1].setAutoFillBackground(False)
      self.leds[-1]['cbox'][-1].setStyleSheet(_fromUtf8("background-color: rgb(21, 236, 255);"))

      # 2nd row
      self.leds[-1]['cbox'].append(QtGui.QGroupBox(self.leds[-1]['container']))
      self.leds[-1]['cbox'][-1].setGeometry(QtCore.QRect(xoff+self.CBOX_XOFFSET,\
        yoff+self.CBOX_YOFFSET[1],self.CBOX_WIDTH,self.CBOX_HEIGHT))
      self.leds[-1]['cbox'][-1].setAutoFillBackground(False)
      self.leds[-1]['cbox'][-1].setStyleSheet(_fromUtf8("background-color: rgb(21, 236, 255);"))

      self.verticalLayout.addWidget(self.leds[-1]['container'])
      
    # other stuff
    self.pushButton = QtGui.QPushButton(self.centralwidget)
    self.pushButton.setGeometry(QtCore.QRect())
    self.pushButton.setText('Quit')
    self.pushButton.clicked.connect(MainWindow.close)
    #QtCore.QObject.connect(self.pushButton, QtCore.SIGNAL(_fromUtf8("clicked()")), MainWindow.close)
    self.verticalLayout1.addWidget(self.pushButton)
    self.scrollArea.setWidget(self.ledwidget)

    # build right panel
    self.verticalLayout2 = QtGui.QVBoxLayout()
    self.horizontalLayout1.addLayout(self.verticalLayout2)
    self.verticalLayout2.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
    
    self.BUTTON_WIDTH=int(200*(1.0*self.DPI_SCALING/2))
    self.BUTTON_HEIGHT=int(50*(1.0*self.DPI_SCALING/2))
    self.saveButton=QtGui.QPushButton()
    self.saveButton.setGeometry(QtCore.QRect(0,0,self.BUTTON_WIDTH,self.BUTTON_HEIGHT))
    self.saveButton.setText('Save')
    self.saveButton.clicked.connect(self.save)
    #QtCore.QObject.connect(self.saveButton, QtCore.SIGNAL(_fromUtf8("clicked()")), self.save)
    self.verticalLayout2.addWidget(self.saveButton)
    
    self.loadButton=QtGui.QPushButton()
    self.loadButton.setGeometry(QtCore.QRect(0,0,self.BUTTON_WIDTH,self.BUTTON_HEIGHT))
    self.loadButton.setText('Load')
    self.loadButton.clicked.connect(self.load)
    #QtCore.QObject.connect(self.loadButton, QtCore.SIGNAL(_fromUtf8("clicked()")), self.load)
    self.verticalLayout2.addWidget(self.loadButton)
    
    self.exportButton=QtGui.QPushButton()
    self.exportButton.setGeometry(QtCore.QRect(0,0,self.BUTTON_WIDTH,self.BUTTON_HEIGHT))
    self.exportButton.setText('Export')
    self.exportButton.clicked.connect(self.export)
    #QtCore.QObject.connect(self.exportButton, QtCore.SIGNAL(_fromUtf8("clicked()")), self.export)
    self.verticalLayout2.addWidget(self.exportButton)
    
    # set up RTC check box and button
    self.layoutRTC=QtGui.QHBoxLayout()
    self.layoutRTC.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft)
    self.verticalLayout2.addLayout(self.layoutRTC)
    
    self.checkRTC=QtGui.QCheckBox()
    self.checkRTC.setGeometry(QtCore.QRect(xoff,yoff,\
      self.COPY_CHECK_WIDTH,self.COPY_CHECK_HEIGHT))
    self.checkRTC.setChecked(True)
    self.layoutRTC.addWidget(self.checkRTC)
    
    self.labelRTC=QtGui.QLabel()
    self.labelRTC.setGeometry(QtCore.QRect(xoff,yoff,\
      self.LABEL_WIDTH1,self.LABEL_HEIGHT))
    self.labelRTC.setText(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">Include RTC?</span></p></body></html>", None))
    self.layoutRTC.addWidget(self.labelRTC)

    self.preview=QtGui.QLabel()
    self.preview.setGeometry(QtCore.QRect(0,0,int(390*(1.0*self.DPI_SCALING/2)),int(100*(1.0*self.DPI_SCALING/2))))
    self.preview.setText(_translate("MainWindow", "<html><head/><body>\
      <p><span style=\" font-size:10pt;color:#777777;font-style:normal;\">Matrix Preview</span></p>\
      </body></html>", None))
    self.verticalLayout2.addWidget(self.preview)

    self.PAINTAREA_SIZE=int(370*(1.0*self.DPI_SCALING/2))
    self.qpArea2=PaintArea(self.LED_NUMBER,self)
    self.qpArea2.setMinimumSize(QtCore.QSize(self.PAINTAREA_SIZE,self.PAINTAREA_SIZE))
    self.qpArea2.setMaximumSize(QtCore.QSize(self.PAINTAREA_SIZE,self.PAINTAREA_SIZE))
    self.qpArea2.setCursor(QtCore.Qt.PointingHandCursor)
    self.verticalLayout2.addWidget(self.qpArea2)
    
    self.about=QtGui.QLabel()
    self.about.setGeometry(QtCore.QRect(0,0,int(390*(1.0*self.DPI_SCALING/2)),int(100*(1.0*self.DPI_SCALING/2))))
    self.about.setText(_translate("MainWindow", "<html><head/><body>\
      <p><span style=\" font-size:12pt;\">LED Matrix v0.1</span></p>\
      <p><span style=\" font-size:10pt;\">by M&ouml;glich lab 2017</span></p>\
      <p><span style=\" font-size:10pt;\">University of Bayreuth</span></p></body></html>", None))
    self.verticalLayout2.addWidget(self.about)

    # set up connectors
    for currled in range(self.LED_NUMBER):
      # connect events
      self.leds[currled]['qradio'][-2].toggled.connect(partial(self.disable_entry,self.leds[currled]['qedit'][-1],True))
      #QtCore.QObject.connect(self.leds[currled]['qradio'][-2], QtCore.SIGNAL(_fromUtf8("toggled(bool)")),partial(self.disable_entry,self.leds[currled]['qedit'][-1],True))
      self.leds[currled]['qradio'][-1].toggled.connect(partial(self.disable_entry,self.leds[currled]['qedit'][-1],False))
      #QtCore.QObject.connect(self.leds[currled]['qradio'][-1], QtCore.SIGNAL(_fromUtf8("toggled(bool)")),partial(self.disable_entry,self.leds[currled]['qedit'][-1],False))

      # edit fields => update color
      for i in range(3):
        self.leds[currled]['qedit'][i].textChanged.connect(partial(self.update_color,self.leds[currled]['cbox'][0],currled,1))
        #QtCore.QObject.connect(self.leds[currled]['qedit'][i], QtCore.SIGNAL(_fromUtf8("textChanged(QString)")),partial(self.update_color,self.leds[currled]['cbox'][0],currled,1))
      self.update_color(self.leds[currled]['cbox'][0],currled,1)
      for i in range(3):
        self.leds[currled]['qedit'][i+4].textChanged.connect(partial(self.update_color,self.leds[currled]['cbox'][1],currled,2))
        #QtCore.QObject.connect(self.leds[currled]['qedit'][i+4], QtCore.SIGNAL(_fromUtf8("textChanged(QString)")),partial(self.update_color,self.leds[currled]['cbox'][1],currled,2))
      self.update_color(self.leds[currled]['cbox'][1],currled,2)

      # copy button
      self.leds[currled]['check'][1].clicked.connect(partial(self.copy,currled))
      #QtCore.QObject.connect(self.leds[currled]['check'][1], QtCore.SIGNAL(_fromUtf8("clicked()")), partial(self.copy,currled))

    # main window
    MainWindow.setCentralWidget(self.centralwidget)
    MainWindow.setWindowTitle(_translate("MainWindow", "LED Matrix", None))
    self.CENTRAL_WIDTH=int(1280*(1.0*self.DPI_SCALING/2))
    self.CENTRAL_HEIGHT=int(800*(1.0*self.DPI_SCALING/2))
    self.centralwidget.setMinimumSize(QtCore.QSize(self.CENTRAL_WIDTH,self.CENTRAL_HEIGHT))
    self.centralwidget.setMaximumSize(QtCore.QSize(self.CENTRAL_WIDTH,16777215))
    self.leds[0]['qedit'][0].setFocus()
    self.leds[0]['qedit'][0].selectAll()
    
  def disable_entry(self,target=None,value=False):
    if (target):
      target.setEnabled(value)
    
  def update_color(self,target=None,currled=0,color=1):
    # update cbox entry
    rgb=[self.leds[currled]['qedit'][i+(color-1)*4].text() for i in range(3)]
    rgb=[str(i) for i in rgb]
    rgb=[i if (len(i)) else '0' for i in rgb]
    colstr=','.join(rgb)
    target.setStyleSheet(_fromUtf8("background-color: rgb("+colstr+");"))
    
    # update LED matrix cartoon
    if (color==1):
      self.qpArea2.update_color(currled,[int(i) for i in rgb])
      
  def save(self):
    # does what it says
    filename=QtGui.QFileDialog.getSaveFileName(filter='Config Files (*.cfg);;Text Files (*.txt);;All files (*.*)')
    filename=str(filename)
    
    # now save parameters to file
    try:
      writehandle=open(filename,'w')
      
      if(writehandle):
        # write general stuff
        writehandle.write('<no. LEDs>'+str(self.LED_NUMBER)+'</no. LEDs>\n')
        
        for i in range(self.LED_NUMBER):
          writehandle.write('<LED '+str(i)+'>\n')
          writehandle.write('\t<mode>'+str(int(self.leds[i]['qradio'][2].isChecked()))+'</mode>\n')
          col1=[str(self.leds[i]['qedit'][j].text()) for j in range(3)]
          col1=','.join([j if(len(j)) else '0' for j in col1])
          writehandle.write('\t<color1>'+col1+'</color1>\n')
          col2=[str(self.leds[i]['qedit'][4+j].text()) for j in range(3)]
          col2=','.join([j if(len(j)) else '0' for j in col2])
          writehandle.write('\t<color2>'+col2+'</color2>\n')
          times=[str(self.leds[i]['qedit'][j].text()) for j in (3,7)]
          times=[j if (len(j)) else '0' for j in times]
          writehandle.write('\t<time1>'+times[0]+'</time1>\n')
          writehandle.write('\t<time2>'+times[1]+'</time2>\n')
          writehandle.write('</LED '+str(i)+'>\n')
        
        writehandle.close()
    except:
      pass

  def load(self):
    # does what it says
    filename=QtGui.QFileDialog.getOpenFileName(filter='Config Files (*.cfg);;Text Files (*.txt);;All files (*.*)')
    filename=str(filename)
    
    # now read parameters from file
    try:
      readhandle=open(filename,'r')
      
      if(readhandle):
        # parse entire file
        parsed=readhandle.readlines()
        readhandle.close()
        parsed=''.join(parsed)
        parsed=re.sub('\s','',parsed)
        
        # search for number of LEDs
        mt=re.match('.*<no\.LEDs>(.+)</no\.LEDs>',parsed)
        if (mt):
          noleds=int(mt.group(1))
          for i in range(noleds):
            matchstring='.*<LED'+str(i)+'>(.+)</LED'+str(i)+'>'
            mt=re.match(matchstring,parsed)#,re.DOTALL)
            if (mt):
              parse2=mt.group(1)
              
              # read out mode
              mt=re.match('.*<mode>(.+)</mode>',parse2)
              if (mt):
                if (bool(int(mt.group(1)))):
                  self.leds[i]['qradio'][2].setChecked(True)
                else:
                  self.leds[i]['qradio'][1].setChecked(True)
  
              # read out color1
              mt=re.match('.*<color1>(.+)</color1>',parse2)
              if (mt):
                self.rgb=mt.group(1).split(',')
                for j in range(3):
                  self.leds[i]['qedit'][j].setText(self.rgb[j])
  
              # read out color2
              mt=re.match('.*<color2>(.+)</color2>',parse2)
              if (mt):
                self.rgb=mt.group(1).split(',')
                for j in range(3):
                  self.leds[i]['qedit'][j+4].setText(self.rgb[j])
  
              # read out time1
              mt=re.match('.*<time1>(.+)</time1>',parse2)
              if (mt):
                self.leds[i]['qedit'][3].setText(mt.group(1))
  
              # read out time2
              mt=re.match('.*<time2>(.+)</time2>',parse2)
              if (mt):
                self.leds[i]['qedit'][7].setText(mt.group(1))
    except:
      pass
              
  def copy(self,source=0):
    # copies information from clicked entry to all other entries that are checked
    # retrieve values from source LED
    radio=self.leds[source]['qradio'][1].isChecked()
    color1=[str(self.leds[source]['qedit'][j].text()) for j in range(3)]
    color2=[str(self.leds[source]['qedit'][j+4].text()) for j in range(3)]
    time1=str(self.leds[source]['qedit'][3].text())
    time2=str(self.leds[source]['qedit'][7].text())
    
    # cycle over all entries
    for i in range(self.LED_NUMBER):
      if (self.leds[i]['check'][0].isChecked()):
        # copy mode
        if (radio):
          self.leds[i]['qradio'][1].setChecked(True)
        else:
          self.leds[i]['qradio'][2].setChecked(True)

        # copy color1
        for j in range(3):
          self.leds[i]['qedit'][j].setText(color1[j])

        # copy color2
        for j in range(3):
          self.leds[i]['qedit'][j+4].setText(color2[j])

        # copy time1
        self.leds[i]['qedit'][3].setText(time1)

        # copy time2
        self.leds[i]['qedit'][7].setText(time2)
        
  def select_LED(self,currled=0):
    #print currled
    self.leds[currled]['qedit'][0].setFocus()
    self.leds[currled]['qedit'][0].selectAll()
    self.scrollArea.ensureWidgetVisible(self.leds[currled]['qedit'][0])

  def export(self):
    # exports LED configuration to ino file
    filename=QtGui.QFileDialog.getSaveFileName(filter='Arduino Sketch (*.ino);;All files (*.*)')
    filename=str(filename)
    
    # source code of Arduino File
    arduino=\
'''// define whether RTC is to be used or not
#define USE_RTC <<<RTC_SETTING>>>

// includes for RTC
#if USE_RTC
  #include <Wire.h>
  #include "RTClib.h"
#endif

// include for NeoPixel matrix
#include <Adafruit_NeoPixel.h>

/*
class PixelTimer
which is an object that should make it easy
to programm a large number of neo pixels to
turn on/off at various intervals unrelated
to each other

Florian Richter, Moeglich Lab HU Berlin
florian.richter.1@hu-berlin.de, feb 2014

extended by the incredible AM
andreas.moeglich@uni-bayreuth.de

*/

unsigned int neopixel_pin  = 6;
#define  num_neopixels <<<LED_NUMBER>>>
#define PI 3.14159265358979323846
Adafruit_NeoPixel led_matrix = Adafruit_NeoPixel(num_neopixels, neopixel_pin, NEO_GRB + NEO_KHZ800);

void component_colors_from_uint32t(
  uint32_t color,
  uint8_t & red_val,
  uint8_t & green_val,
  uint8_t & blue_val
 )
 {
  red_val = (uint8_t)(color >> 16),
  green_val = (uint8_t)(color >>  8),
  blue_val = (uint8_t)color;
 }

boolean is_led_set(
  uint8_t which_led, uint8_t set_red, uint8_t set_green, uint8_t set_blue )
 {
 
  uint32_t current_status(  led_matrix.getPixelColor( which_led ) );
  
  uint8_t cur_r, cur_g, cur_b;
  component_colors_from_uint32t( current_status, cur_r, cur_g, cur_b );
  if( (cur_r == set_red ) && (cur_g == set_green ) && (cur_b == set_blue ) ){
    return true;
  }
  else{
    return false;
  }
 }

class PixelTimer {

public:
  PixelTimer();
  
  PixelTimer( 
    uint8_t which_pixel,
    uint8_t intensity_r,
    uint8_t intensity_g,
    uint8_t intensity_b,
    uint8_t intensity_r2,
    uint8_t intensity_g2,
    uint8_t intensity_b2,
    uint8_t mode,
    unsigned long param1, 
    unsigned long param2
  );

  ~PixelTimer();
  
  void set_values(
    uint8_t which_pixel,
    uint8_t intensity_r,
    uint8_t intensity_g,
    uint8_t intensity_b,
    uint8_t intensity_r2,
    uint8_t intensity_g2,
    uint8_t intensity_b2,
    uint8_t mode,
    unsigned long param1, 
    unsigned long param2 
  );
  
  void process_time( unsigned long current_time );
  void init_on();

private:
  uint8_t my_pixel_;
  uint8_t intensity_r_;
  uint8_t intensity_g_;
  uint8_t intensity_b_;
  uint8_t intensity_r2_;
  uint8_t intensity_g2_;
  uint8_t intensity_b2_;
  uint8_t mode_;
  unsigned long param1_, param2_;

  void PixelTimer::boxcar (unsigned long current_time);
  void PixelTimer::sine (unsigned long current_time);
};

PixelTimer::PixelTimer() : my_pixel_(num_neopixels +1), intensity_r_(0), intensity_g_(0), intensity_b_(0), intensity_r2_(0), intensity_g2_(0), intensity_b2_(0), mode_(0), param1_(0), param2_(0)
{}

PixelTimer::PixelTimer(
  uint8_t which_pixel,
  uint8_t intensity_r,
  uint8_t intensity_g,
  uint8_t intensity_b,
  uint8_t intensity_r2,
  uint8_t intensity_g2,
  uint8_t intensity_b2,
  uint8_t mode,
  unsigned long param1,
  unsigned long param2
) : my_pixel_(which_pixel), intensity_r_(intensity_r), intensity_g_(intensity_g), intensity_b_(intensity_b), intensity_r2_(intensity_r2), intensity_g2_(intensity_g2), intensity_b2_(intensity_b2), mode_(mode), param1_(param1), param2_(param2)
{}

PixelTimer::~PixelTimer(){}

void PixelTimer::set_values(
  uint8_t which_pixel,
  uint8_t intensity_r,
  uint8_t intensity_g,
  uint8_t intensity_b,
  uint8_t intensity_r2,
  uint8_t intensity_g2,
  uint8_t intensity_b2,
  uint8_t mode,
  unsigned long param1, 
  unsigned long param2 
)
{
  my_pixel_= which_pixel; 
  intensity_r_ = intensity_r;
  intensity_g_ = intensity_g;
  intensity_b_ = intensity_b;
  intensity_r2_ = intensity_r2;
  intensity_g2_ = intensity_g2;
  intensity_b2_ = intensity_b2;
  mode_ = mode;
  param1_ = param1;
  param2_ = param2;
}

void PixelTimer::init_on()
{
  led_matrix.setPixelColor( my_pixel_, intensity_r_, intensity_g_, intensity_b_);
}


void PixelTimer::process_time(unsigned long current_time )
{
  // switch according to mode
  switch (mode_)
  {
    case 0:
      boxcar(current_time);
      break;
    case 1:
      sine(current_time);
      break;
    default:
      boxcar(current_time);
  }
}

void PixelTimer::boxcar (unsigned long current_time)
{
  unsigned long t_in_current_wave = current_time % (param1_+param2_);

  // check whether we are in on or off phase
  if (t_in_current_wave <= param1_)
  {
    // check LED status
    if (!is_led_set(my_pixel_,intensity_r_, intensity_g_, intensity_b_))
    {
      led_matrix.setPixelColor( my_pixel_, intensity_r_, intensity_g_, intensity_b_);
    }
  }
  else
  {
    // check LED status
    if (!is_led_set(my_pixel_,intensity_r2_, intensity_g2_, intensity_b2_))
    {
      led_matrix.setPixelColor( my_pixel_, intensity_r2_, intensity_g2_, intensity_b2_);
    }
  }
}

void PixelTimer::sine( unsigned long current_time)
{
  unsigned long t_in_current_wave = current_time % param1_;
  double sval;

  sval=pow(sin(2.0*PI*(1.0*t_in_current_wave)/(1.0*param1_)),2);
  led_matrix.setPixelColor( my_pixel_, sval*intensity_r_+(1-sval)*intensity_r2_, sval*intensity_g_+(1-sval)*intensity_g2_, sval*intensity_b_+(1-sval)*intensity_b2_);
}

// global variables
PixelTimer ptimers[num_neopixels];
#if USE_RTC
  RTC_DS3231 rtc;
#endif
// check pixels every how many ms?
int delay_time = 25;
long start_sec;
unsigned long start_millis;
#if USE_RTC
  // specify at which interval (in s) the internal clock is synchronized by the RTC
  long time_correct_interval = 10;
  long prev_sec;
#endif

void setup(){
  // set up NeoPixel matrix
  led_matrix.begin(); // Initialize all pixels  
  // arguments are: pixel ID, red, green, blue, red2, green2, blue2, mode, param1, param2
<<<LED_DEFINITIONS>>>

#if USE_RTC
  // set up RTC and reset time
  rtc.begin();
  DateTime now = rtc.now();
  start_sec=now.secondstime();
  prev_sec=0;
#endif

  // record start of millis
  start_millis=millis();
} 

void loop() {
   unsigned long curr_millis = millis();
#if USE_RTC
   DateTime now = rtc.now();
   long curr_sec=now.secondstime()-start_sec;

   // synchronize millisecond counter to the RTC
   if (curr_sec-time_correct_interval>=prev_sec)
   {
     prev_sec=curr_sec;
     start_millis=curr_millis;
   }
   // only keep the last millis since last update
   curr_millis+=1000*time_correct_interval*floor(curr_sec/time_correct_interval);
   curr_millis-=start_millis;
#endif

   for( int i = 0; i < num_neopixels; i++){
     ptimers[i].process_time( curr_millis );
   }
   led_matrix.show();
   delay( delay_time );
}'''
    
    # now write .ino file
    try:
      writehandle=open(filename,'w')
      if(writehandle):
        # process the output program
        arduino=re.sub('<<<LED_NUMBER>>>',str(self.LED_NUMBER),arduino,re.DOTALL)
        arduino=re.sub('<<<RTC_SETTING>>>',str(int(self.checkRTC.isChecked())),arduino,re.DOTALL)
        
        outstring=''
        for i in range(self.LED_NUMBER):
          params=[self.leds[i]['qedit'][j].text() for j in range(8)]
          params=[int(j) if (len(j)) else 0 for j in params]
          outstring+='  ptimers[%d].set_values(%d, %d, %d, %d, %d, %d, %d, %d, %d, %d);\n'\
            %(i,i,params[0],params[1],params[2],\
            params[4],params[5],params[6],\
            int(self.leds[i]['qradio'][2].isChecked()),params[3],params[7])
        
        arduino=re.sub('<<<LED_DEFINITIONS>>>',outstring,arduino,re.DOTALL)
        
        # write the output file
        writehandle.write(arduino)
        writehandle.close()
    except:
      pass
    

#
#
#

class MyForm(QtGui.QMainWindow):
  def __init__(self, parent=None):
    QtGui.QWidget.__init__(self, parent)
    self.ui = Ui_MainWindow()
    # adjust DPI scaling
    self.targetDPI = 96
    actualDPI = QtGui.QPaintDevice.logicalDpiX(self)
    self.DPI_SCALING = 1.0*actualDPI/self.targetDPI
    self.ui = Ui_MainWindow()
    self.ui.setupUi(MainWindow = self, DPI_SCALING = self.DPI_SCALING)

if __name__ == "__main__":
  app = QtGui.QApplication(sys.argv)
  # this is the setting you may want to change
  myapp = MyForm()
  myapp.show()
  sys.exit(app.exec_())